$(call onDebugMake,fichier 'commands' inclu)

#-------------------------------------------------------------------------------
# File :
#             All commands for make
#-------------------------------------------------------------------------------


# list of all commands
COMMANDS := clean bigclean ultraclean clear b B doxygen cleandoxygen c affCPP affFiles application common tools objects release configure save saveToFtp savecode check checkd help h

# on declare ces elements comme des phony targets (ce sont des commandes, elles ne correspondent pas a des noms de fichiers)
.PHONY : $(COMMANDS)


#-------------------------------------------------------------------------------
# partial compilation
#-------------------------------------------------------------------------------

objects :: $(sort $(GENERATED_FILES_ALL)) $(sort $(OBJ_ALL)) ;

release ::
	@$(ECHOLINEXX)
	@$(call echoexx,$(COLORTITLE))
	@$(call echoexx,----------------- release is not a command but an option, use : "make release=1" ----------------------$(COLORINIT))
	@$(call echoexx,Vous pouvez ainsi lancer des commandes en mode release : "make release=1 clean")
	@$(ECHOLINEXX)



#-------------------------------------------------------------------------------
# Display functions (used by other commands)
#-------------------------------------------------------------------------------

affBegin ::
	@$(call echoexx,$(COLORBEGIN))
	@$(call echoexx,::::::: compilation on $(SYSTEM) of "$(NAME)" begining :::::::$(COLORINIT))
	@$(ECHOEMPTYXX)

affEnd ::
	@$(ECHOEMPTYXX)
	@$(call echoexx,$(logo0)    $(MAIL))
	@$(call echoexx,$(logo1)    $(URL))
	@$(call echoexx,$(logo2))
	@$(call echoexx,$(logo3))
	@$(call echoexx,$(logo4)    $(COLORTITLE2)==============================================$(COLORINIT))
	@$(call echoexx,$(logo5)    $(COLORTITLE2):::::   $(COLORTITLE) $(if $(UNIX),./)$(EXEC)$(EXECUTABLE_EXTENSION) $(COLORTITLE2)   :::::$(COLORINIT))
	@$(call echoexx,$(logo6)    $(COLORTITLE2)==============================================$(COLORINIT))
	@$(call echoexx,$(logo7))
	@$(call echoexx,$(logo8))
	@$(ECHOLINEXX)
	@$(ECHOEMPTYXX)

affRelease ::
	@$(ECHOEMPTYXX)
	@$(call echoexx,$(COLORNOTES)---- Compilation in release mode ----$(COLORINIT))



#-------------------------------------------------------------------------------
# Commandes de bases
#-------------------------------------------------------------------------------

clean ::
	@$(ECHOLINEXX)
	@$(call echoexx,$(COLORTITLE))
	@$(call echoexx,-----------------clean----------------------$(COLORINIT))
	-$(RMXX) $(call convert_slash_to_system,$(COMPILATION))
	-$(RMDIRXX) $(call convert_slash_to_system,$(COMPIL_DIR)src)
	@$(ECHOLINEXX)
	@$(ECHOEMPTYXX)

bigclean ::
	@$(ECHOLINEXX)
	@$(call echoexx,$(COLORTITLE))
	@$(call echoexx,-----------------bigclean-------------------$(COLORINIT))
	-$(RMXX) $(call convert_slash_to_system,$(COMPILATION))
	-$(RMDIRXX) $(call convert_slash_to_system,$(COMPIL_DIR))
	@$(ECHOLINEXX)
	@$(ECHOEMPTYXX)




#-------------------------------------------------------------------------------
#           SHORTCUT for usual commands
#-------------------------------------------------------------------------------

b :: clean all

B ::
	@$(ECHOLINEXX)
	@$(call echoexx,$(COLORTITLE)-----------------B----------------------$(COLORINIT))
	@$(call echoxx,****** B : comme -B, remove all compiled files and dependacies before to re-compile *****)
	@$(MAKESILENTXX) bigclean; $(MAKEXX) --no-print-directory


# compilation commands which depends of the operating systems
include $(MAKE_INTERN_DIR)$(SYSTEM)/commands.mk



#-------------------------------------------------------------------------------
#             DOXYGEN DOCUMENTATION
#-------------------------------------------------------------------------------

doxygen ::
	@$(ECHOLINEXX)
	@$(call echoexx,$(COLORTITLE)--------------doxygen-------------------$(COLORINIT))
	@$(call echoxx,**** generation of documentation ****)
	@$(GENDOC) $(call convert_slash_to_system,makeCfiles/Doxyfile)
	@$(ECHOLINEXX)
	@$(ECHOEMPTYXX)

cleandoxygen ::
	@$(ECHOLINEXX)
	@$(call echoexx,$(COLORTITLE)--------------cleandoxygen-------------------$(COLORINIT))
	@$(call echoxx,**** remove generated documentation ****)
	@$(RMXX) $(call convert_slash_to_system,./doc_doxygen/html/* ./doc_doxygen/latex/*)
	@$(ECHOLINEXX)
	@$(ECHOEMPTYXX)





#-------------------------------------------------------------------------------
# commands for CONFIGURATION FILES
#    - list of files to compile
#    - librairies info (include path, link options, flags)
#    - ...
#-------------------------------------------------------------------------------

# Warning command when the configuration has change
warningconfigure ::
	@$(call echoexx,$(COLORTITLE)---- WARNING ----$(COLORINIT))
	@$(call echoexx,$(COLORWARNING))
	@$(call echoexx,Your configuration has change - may be you need to recompile all in order to respect the new configuration.)
	@$(ECHOEMPTYXX)
	@$(call echoexx,        use : make clean $(AFF_NEXT_COMMAND) make)
	@$(call echoexx,$(COLORINIT))
	@$(ECHOLINEXX)


configuredependencies :: $(SAVE_CONFIG) $(SAVE_FILES) $(SAVE_SESSION) ;
#	@$(call echoexx,$(COLORTITLE)---- TMP configuredependencies ----$(COLORINIT))

configureinit ::
	@$(call echoexx,$(COLORTITLE)---- configure initialisation ----$(COLORINIT))
	-$(RMDIRXX) "$(call convert_slash_to_system,$(MAKEDIR)$(COMPIL_DIR)makeCfiles)"
	@$(ECHOEMPTYXX)

# "configure" command
configure :: configureinit warningconfigure
	@$(call echoexx,$(COLORTITLE)---- configure ----$(COLORINIT))
	@make --no-print-directory

# when configuration has change, we recreate all configuration files
autoconfigure :: configuredependencies warningconfigure ;



ifeq ($(CONFIGURE),)

$(SAVE_FILES) $(SAVE_CONFIG) $(SAVE_SESSION) : Makefile $(BUILD_INTERN_DIR)* $(BUILD_INTERN_DIR)*/* $(BUILD_INTERN_DIR)*/*/*
	@$(call echoexx,$(COLORTITLE)---- configuration file "$@" need to be updated ----$(COLORINIT))
	@$(RMXX) $(call convert_slash_to_system,"$@")
#INDEPENDENT_COMMANDS_GOAL=1

endif




#-------------------------------------------------------------------------------
# commands to check founded libraries
#-------------------------------------------------------------------------------

check :: configuredependencies
	@$(ECHOLINEXX)
	@$(call echoexx,$(COLORTITLE))
	@$(call echoexx,-----------------check----------------------$(COLORINIT))
	@$(ECHOEMPTYXX) $(foreach i,$(sort $(LIBS_TOCHECK)),\
		$(NEXT_COMMAND)$(call echoexx,$(COLORTITLE2)--- $(i) ---$(COLORINIT))\
		$(if $(call not,$($(i)_INCL))$(call not,$($(i)_LINK)),\
			INCL:$($(i)_INCL) _ LINK:$($(i)_LINK)\
			$(if $(call not,$($(i)_INCL)$($(i)_LINK)),\
				$(call echo_convert,$(COLORWARNING)--- Library not found.$(COLORINIT))\
				,\
				$(if $($(i)_INCL),\
					$(NEXT_COMMAND)$(call echoexx,--- include directory found.)\
					$(NEXT_COMMAND)$(call echoexx,$(COLORWARNING)--- linking shared object not found.$(COLORINIT))\
					,\
					$(NEXT_COMMAND)$(call echoexx,$(COLORWARNING)--- include directory not found.$(COLORINIT))\
					$(NEXT_COMMAND)$(call echoexx,--- linking shared object found.)\
				)\
			)\
			,\
			--- OK\
		)\
	)
	@$(ECHOLINEXX)
	@$(ECHOEMPTYXX)


checkd :: configuredependencies
	@$(ECHOLINEXX)
	@$(call echoexx,$(COLORTITLE))
	@$(call echoexx,-----------------checkd----------------------$(COLORINIT))
	@$(ECHOEMPTYXX) $(foreach i,$(sort $(LIBS_TOCHECK)),\
		$(NEXT_COMMAND)$(call echoexx,$(COLORTITLE2)--- $(i) ---$(COLORINIT))\
		$(NEXT_COMMAND)$(call echoexx,$(COLORTITLE3)include directory :$(COLORINIT) $($(i)_INCL))\
		$(NEXT_COMMAND)$(call echoexx,$(COLORTITLE3)linking option    :$(COLORINIT) $($(i)_LINK))\
		$(NEXT_COMMAND)$(call echoexx,$(COLORTITLE3)        directory :$(COLORINIT) $($(i)_LINK_DIR))\
	)
	@$(ECHOLINEXX)
	@$(ECHOEMPTYXX)



#-------------------------------------------------------------------------------
# Only display commands
#-------------------------------------------------------------------------------

c ::
	$(CLEARXX)

affFiles ::
	@$(ECHOLINEXX)
	@$(call echoexx,$(COLORTITLE)--------------affFiles-------------------$(COLORINIT))
	@$(ECHOEMPTYXX)
	@$(call echoexx,$(COLORTITLE)**** List of files to generate ****$(COLORINIT))
	@$(call echoxx,$(GENERATED_FILES_ALL))
	@$(ECHOEMPTYXX)
	@$(call echoexx,$(COLORTITLE)**** List of all object files to compile ****$(COLORINIT))
	@$(call echoxx,$(OBJ_ALL))
	@$(ECHOEMPTYXX)
	@$(ECHOLINEXX)
	@$(ECHOEMPTYXX)


help h ::
	@$(ECHOLINEXX)
	@$(call echoexx,$(COLORTITLE)--------------help-------------------$(COLORINIT))
	@$(ECHOEMPTYXX)
	@$(call echoexx,$(COLORTITLE3)release=1    $(COLORINIT): compile in release mode (without debug options))
	@$(ECHOEMPTYXX)
	@$(call echoexx,$(COLORTITLE3)clean        $(COLORINIT): remove compiled files from project)
	@$(call echoexx,$(COLORTITLE3)bigclean     $(COLORINIT): remove all compiled files)
	@$(call echoexx,$(COLORTITLE3)ultraclean   $(COLORINIT): remove all compiled files and temporary files)
	@$(call echoexx,$(COLORTITLE3)clear        $(COLORINIT): remove temporary files)
	@$(ECHOEMPTYXX)
	@$(call echoexx,$(COLORTITLE3)b            $(COLORINIT): clean + compile)
	@$(call echoexx,$(COLORTITLE3)B            $(COLORINIT): bigclean + compile)
	@$(ECHOEMPTYXX)
	@$(call echoexx,$(COLORTITLE3)depend       $(COLORINIT): recreate all dependancies files (use only in particular case))
	@$(call echoexx,$(COLORTITLE3)cleandepend  $(COLORINIT): remove all dependancies files (use only in particular case))
	@$(ECHOEMPTYXX)
	@$(call echoexx,$(COLORTITLE3)check        $(COLORINIT): check if all libraries have been founded)
	@$(call echoexx,$(COLORTITLE3)checkd       $(COLORINIT): print include path and link options for each library declared)
	@$(ECHOEMPTYXX)
	@$(call echoexx,$(COLORTITLE3)doxygen      $(COLORINIT): create doxygen documentation (in the directory : ./doc_doxygen/html))
	@$(call echoexx,$(COLORTITLE3)cleandoxygen $(COLORINIT): remove the documentation generate by doxygen)
	@$(ECHOEMPTYXX)
	@$(call echoexx,$(COLORTITLE3)c            $(COLORINIT): clear the terminal)
	@$(call echoexx,$(COLORTITLE3)affFiles     $(COLORINIT): print the list of files)
	@$(ECHOEMPTYXX)
	@$(call echoexx,$(COLORTITLE3)objects      $(COLORINIT): only compile object files)
	@$(ECHOEMPTYXX)
	@$(call echoexx,$(COLORTITLE3)save         $(COLORINIT): create an archive of the project (in the parent directory))
	@$(call echoexx,$(COLORTITLE3)saveToFtp    $(COLORINIT): create an archive and send it to ftp)
	@$(call echoexx,$(COLORTITLE3)savecode     $(COLORINIT): create an archive of the project only with the source code directory "src")
	@$(ECHOEMPTYXX)
	@$(call echoexx,$(COLORTITLE3)debugmake=1  $(COLORINIT): print all actions of the makefile to debug it)
	@$(ECHOEMPTYXX)





#-------------------------------------------------------------------------------
# SECONDARY COMMANDS
#-------------------------------------------------------------------------------

# commands which doesn't be multiprocess (when make is called with -j option)
.NOTPARALLEL += save saveToFtp savecode

DOXYGEN_DIR ?= doc_doxygen
# we remove temporary files from archive ("*~"), hidden files (begin by a point ".*"), ...
ARCHIVE_EXCLUDES ?= '*~' '*$(OBJECT_EXTENSION)' '*/.[^./]*' '*/$(ROOT_COMPIL_DIR)*' '*/$(DOXYGEN_DIR)/html/*' '*$(DOXYGEN_DIR)/latex/*'
ARCHIVE_NAME := $(NAME)_$(DATE)
ARCHIVE_EXTENSION ?= .tar.bz2

ifeq ($(ARCHIVE_EXTENSION),.tar.gz)
  ARCHIVE_COMPRESSION := --gzip
else
ifeq ($(ARCHIVE_EXTENSION),.tar.bz2)
  ARCHIVE_COMPRESSION := --bzip2
endif
endif

save ::
	@$(ECHOLINEXX)
	@$(call echoexx,$(COLORTITLE)------------------save----------------------$(COLORINIT))
	@$(call echoxx, $(ARCHIVE_NAME)$(ARCHIVE_EXTENSION)     :    $(CURRENTDIR))
	tar -c -v --totals $(ARCHIVE_COMPRESSION) -f ../$(ARCHIVE_NAME)$(ARCHIVE_EXTENSION) --exclude-caches $(ARCHIVE_EXCLUDES:%=--exclude=%) ../$(CURRENTDIR)
	@$(ECHOLINEXX)
	@$(ECHOEMPTYXX)

savecode ::
	@$(ECHOLINEXX)
	@$(call echoexx,$(COLORTITLE)------------------savecode----------------------$(COLORINIT))
	@$(call echoxx, $(ARCHIVE_NAME)_SRC$(ARCHIVE_EXTENSION)     :    $(CURRENTDIR))
	tar -c -v --totals $(ARCHIVE_COMPRESSION) -f ../$(ARCHIVE_NAME)_SRC$(ARCHIVE_EXTENSION) --exclude-caches $(ARCHIVE_EXCLUDES:%=--exclude=%) ../$(CURRENTDIR)/src
	@$(ECHOLINEXX)
	@$(ECHOEMPTYXX)

saveToFtp :: save
	@$(ECHOLINEXX)
	@$(call echoexx,$(COLORTITLE)------------------saveToFtp----------------------$(COLORINIT))
	@lftp -e "cd $(FTP_DIR) ;  put ../$(ARCHIVE_NAME)$(ARCHIVE_EXTENSION) ; quit" "$(FTP)"

#commitToFtp ::
#	@$(ECHOLINEXX)
#	@$(call echoexx,$(COLORTITLE)------------------commitToFtp----------------------$(COLORINIT))
#	@lftp -e "cd CODE ;  put $(shell $(FILES_TO_SAVE) ) ; quit" "$(FTP)"
#
#doxygenToFtp ::
#	@$(ECHOLINEXX)
#	@$(call echoexx,$(COLORTITLE)------------------doxygenToFtp----------------------$(COLORINIT))
#	@$(call echoexx,$(DOXYGEN_FILES_TO_SAVE))
#	@lftp "$(FTP)" -e "cd CODE/$(CURRENTDIR) ;" -e "  mput -d $(shell $(DOXYGEN_FILES_TO_SAVE) -not \( -name "*.h*" -o -name "*.e*" \) ) ;"
#	@lftp queue -e "mput -d $(shell $(DOXYGEN_FILES_TO_SAVE) -name "*.h*" -o -name "*.e*" ) ; quit"



