$(call onDebugMake,fichier 'create' inclu)

#-------------------------------------------------------------------------------
# File :
#        Functions to create rules for the compilation of :
#          - objects
#          - executable
#          - static library
#          - dynamic library
#-------------------------------------------------------------------------------


# creer les regles de compilation automatiques
#
# $1 : LIBS, noms des variables contenant les infos sur les librairies utilisees
# $2 : FLAGS, les flags de compilation (ex: -O3)
# $3 : INCLUDES, les repertoires (ex: mydir/mydir)
#
# exemple : $(call create_rules, LIBS, $(FLAGS), $(DIRECTORIES))
#
create_rules = $(3ArgFunction)                                                 \
	$(eval include $(MAKE_INTERN_DIR)rules.mk)                                 \
	$(eval include $(MAKE_INTERN_DIR)rulesWithFlags.mk)                        \
	$(eval __CREATE_RULES__:=1)                                                \
	$(eval RULES_FLAGS   :=$(2) $(foreach i,$(1),$($(i)_FLAG)))                \
	$(eval RULES_INCLUDEPATH :=$(3) $(foreach i,$(1),$($(i)_INCL)))            \
	$(eval $(call private_rules_with_specific_flags,$(RULES_FLAGS),$(RULES_INCLUDEPATH),$(COMPIL_DIR)))

# creer les regles de compilation automatiques
#
# $1 : OBJS, noms des variables contenant la liste des objets
# $2 : LIBS, noms des variables contenant les infos sur les librairies utilisees
# $3 : FLAGS, les flags de compilation
# $4 : INCLUDES, les repertoires
#
# exemple : $(call create_rules_with_specific_flags, OBJS_APPLICATION, LIBS, $(FLAGS), $(DIRECTORIES))
#
create_rules_with_specific_flags = $(4ArgFunction)                             \
  $(eval include $(MAKE_INTERN_DIR)rules.mk)                                   \
  $(eval include $(MAKE_INTERN_DIR)rulesWithFlags.mk)                          \
  $(eval __CREATE_RULES_WITH_SPECIFIC_FLAGS__:=1)                              \
  $(eval RULES_DIRS:=$(sort $(foreach i,$(1),$($(i)_DIR))))                    \
  $(foreach basedir,$(RULES_DIRS),                                             \
    $(eval RULES_DIR=$(basedir))                                               \
    $(eval RULES_FLAGS:=$(3) $(foreach i,$(1),$($(i)_FLAG)))                   \
    $(eval RULES_INCLUDEPATH:=$(4) $(basedir) $(foreach i,$(2),$($(i)_INCL)))  \
    $(eval $(call private_rules_with_specific_flags,$(RULES_FLAGS),$(RULES_INCLUDEPATH),$(COMPIL_DIR),$(RULES_DIR))) \
  )


# creer les regles de compilation automatiques
#
# $1 : LIBS, noms des variables contenant les infos sur les librairies utilisees
# $2 : FLAGS, les flags de compilation (ex: -O3)
# $3 : INCLUDES, les repertoires (ex: mydir/mydir)
#
# exemple : $(call create_rules LIBS, $(FLAGS), $(DIRECTORIES))
#
create_rules_for_generated_files = $(4ArgFunction)                             \
	$(eval include $(MAKE_INTERN_DIR)rules.mk)                                 \
	$(eval include $(MAKE_INTERN_DIR)rulesWithFlags.mk)                        \
	$(eval __CREATE_RULES_FOR_GENERATED_FILES__:=1)                            \
	$(eval RULES_FLAGS   :=$(2) $(foreach i,$(1),$($(i)_FLAG)))                \
	$(eval RULES_INCLUDEPATH :=$(3) $(foreach i,$(1),$($(i)_INCL)))            \
	$(eval $(call private_rules_with_specific_flags,$(RULES_FLAGS),$(RULES_INCLUDEPATH)))







# creer les regles pour compiler un executable
#
# $1 : EXEC, nom de l'executable
# $2 : OBJS, noms des variables contenant la liste des objets
# $3 : LIBS, noms des variables contenant les infos sur les librairies utilisees
# $4 : FLAGS, les flags de compilation
# $5 : LINK, les repertoires
#
# example : $(call create_exec,myApplication,OBJ_COMMON,MYLIBS,FLAGS)
#

create_exec = $(5ArgFunction) \
	$(eval $(call private_create_exec,$(1),$(2),$(3),$(4),$(5)))

define private_create_exec

$(1) : $(sort $(foreach i,$(2),$($(i)_GENERATED_FILES))) $(sort $(foreach i,$(2),$($(i))))
	@$(call echoexx,$(COLORTITLE)---- Compile exec : $(1)$(EXECUTABLE_EXTENSION) ----$(COLORINIT))
	$(LINKXX) \
	\
	$(LINKXX_FLAG_EXECUTABLE) \
	\
	$(4) \
	$(foreach i,$(3),$($(i)_FLAG)) \
	\
	$(foreach i,$(2),$($(i):%="%")) \
	\
	$(CXX_FLAG_EXEC_OUTPUT)"$(1)$(EXECUTABLE_EXTENSION)" \
	\
	$(foreach i,$(3),$($(i)_LINK:%=$(LINKXX_FLAG_LINK_LIBRARY)%$(LINKXX_FLAG_LIBRARY_EXTENSION))) \
	$(subst $(SPACE_FAKE),$(SPACE), \
		$(foreach i,$(3),$($(i)_LINK_DIR:%=$(LINKXX_FLAG_LINK_DIRECTORY)"%")) \
		$(if $(findstring $(COMPILATOR),gcc), \
			$(if $(foreach i,$(3),$($(i)_LINK_STATIC)), $(LINKXX_FLAG_LINK_LIBRARY_STATIC) $(foreach i,$(3),$($(i)_LINK_STATIC:%=$(LINKXX_FLAG_LINK_LIBRARY)%)),) \
			$(if $(foreach i,$(3),$($(i)_LINK_DYNAMIC)), $(LINKXX_FLAG_LINK_LIBRARY_DYNAMIC) $(foreach i,$(3),$($(i)_LINK_DYNAMIC:%=$(LINKXX_FLAG_LINK_LIBRARY)%)),) \
			\
			$(if $(foreach i,$(3),$($(i)_LINK_DIR)), $(LINKXX_FLAG_SAVE_LINK_PATH)$(subst $(SPACE),:,$(foreach i,$(3),$($(i)_LINK_DIR:%="%"))),)\
		) \
		$(if $(findstring $(COMPILATOR),visualc), \
			$(foreach i,$(3),$($(i)_LINK_STATIC:%=$(LINKXX_FLAG_LINK_LIBRARY_STATIC)%$(LINKXX_FLAG_LIBRARY_EXTENSION))) \
			$(foreach i,$(3),$($(i)_LINK_DYNAMIC:%=$(LINKXX_FLAG_LINK_LIBRARY_DYNAMIC)%$(LINKXX_FLAG_SHARED_LIBRARY_EXTENSION))) \
		) \
	)
	@$(ECHOLINEXX)

endef

#	\
	$(subst $(SPACE_FAKE),$(SPACE),\
		$(5:%=$(CXX_FLAG_INCLUDE_DIRECTORY)"%") \
		$(foreach i,$(3),$($(i)_INCL:%=$(CXX_FLAG_INCLUDE_DIRECTORY)"%")) \
	)\


# creer les regles pour compiler une librairie dynamique
#
# $1 : LIB, nom de la librairie complet (avec le chemin et l'extension)
# $2 : OBJS, noms des variables contenant la liste des objets
# $3 : LIBS, noms des variables contenant les infos sur les librairies utilisees
# $4 : FLAGS, les flags de compilation
# $5 : LINK, les repertoires
#
# example : $(call create_dynamic_lib,directory/mylib.so,OBJ_COMMON,MYLIBS,FLAGS)
#

create_dynamic_lib = $(5ArgFunction) \
	$(eval $(call private_create_dynamic_lib,$(1),$(2),$(3),$(4),$(5)))

define private_create_dynamic_lib

$(1) : $(sort $(foreach i,$(2),$($(i)_GENERATED_FILES))) $(sort $(foreach i,$(2),$($(i))))
	@$(call echoexx,$(COLORTITLE)---- Compile dynamic library : $(1) ----$(COLORINIT))
	$(LINKXX) \
	\
	$(LINKXX_FLAG_SHARED_LIBRARY) \
	\
	$(4) \
	$(foreach i,$(3),$($(i)_FLAG)) \
	\
	$(subst $(SPACE_FAKE),$(SPACE),\
		$(5:%=$(CXX_FLAG_INCLUDE_DIRECTORY)"%") \
		$(foreach i,$(3),$($(i)_INCL:%=$(CXX_FLAG_INCLUDE_DIRECTORY)"%")) \
	)\
	\
	$(foreach i,$(2),$($(i):%="%")) \
	\
	$(CXX_FLAG_LIB_OUTPUT)"$(1)" \
	\
	$(foreach i,$(3),$($(i)_LINK:%=$(LINKXX_FLAG_LINK_LIBRARY)%$(LINKXX_FLAG_LIBRARY_EXTENSION))) \
	$(subst $(SPACE_FAKE),$(SPACE), \
		$(foreach i,$(3),$($(i)_LINK_DIR:%=$(LINKXX_FLAG_LINK_DIRECTORY)"%")) \
		$(if $(findstring $(COMPILATOR),gcc), \
			$(if $(foreach i,$(3),$($(i)_LINK_STATIC)), $(LINKXX_FLAG_LINK_LIBRARY_STATIC) $(foreach i,$(3),$($(i)_LINK_STATIC:%=$(LINKXX_FLAG_LINK_LIBRARY)%)),) \
			$(if $(foreach i,$(3),$($(i)_LINK_DYNAMIC)), $(LINKXX_FLAG_LINK_LIBRARY_DYNAMIC) $(foreach i,$(3),$($(i)_LINK_DYNAMIC:%=$(LINKXX_FLAG_LINK_LIBRARY)%)),) \
			\
			$(if $(foreach i,$(3),$($(i)_LINK_DIR)), $(LINKXX_FLAG_SAVE_LINK_PATH)$(subst $(SPACE),:,$(foreach i,$(3),$($(i)_LINK_DIR:%="%"))),)\
		) \
		$(if $(findstring $(COMPILATOR),visualc), \
			$(foreach i,$(3),$($(i)_LINK_STATIC:%=$(LINKXX_FLAG_LINK_LIBRARY_STATIC)%$(LINKXX_FLAG_LIBRARY_EXTENSION))) \
			$(foreach i,$(3),$($(i)_LINK_DYNAMIC:%=$(LINKXX_FLAG_LINK_LIBRARY_DYNAMIC)%$(LINKXX_FLAG_SHARED_LIBRARY_EXTENSION))) \
		) \
	)
	@$(ECHOLINEXX)

endef



# creer les regles pour compiler une librairie statique
#
# $1 : LIB, nom de la librairie complet (avec le chemin et l'extension)
# $2 : OBJS, noms des variables contenant la liste des objets
# $3 : LIBS, noms des variables contenant les infos sur les librairies utilisees
# $4 : FLAGS, les flags de compilation
# $5 : LINK, les repertoires
#
# example : $(call create_static_lib,directory/mylib.a,OBJ_COMMON,MYLIBS,FLAGS)
#

create_static_lib = $(5ArgFunction) \
	$(eval $(call private_create_static_lib,$(1),$(2),$(3),$(4),$(5)))

define private_create_static_lib

$(1) : $(sort $(foreach i,$(2),$($(i)_GENERATED_FILES))) $(sort $(foreach i,$(2),$($(i))))
	@$(call echoexx,$(COLORTITLE)---- Compile static library : $(1) ----$(COLORINIT))
	$(LIBXX)  \
	\
	$(4) \
	$(foreach i,$(3),$($(i)_FLAG)) \
	\
	$(subst $(SPACE_FAKE),$(SPACE),\
		$(5:%=$(CXX_FLAG_INCLUDE_DIRECTORY)"%") \
		$(foreach i,$(3),$($(i)_INCL:%=$(CXX_FLAG_INCLUDE_DIRECTORY)"%")) \
	)\
	\
	$(foreach i,$(2),$($(i):%="%")) \
	\
	$(CXX_FLAG_LIB_OUTPUT)"$(1)" \
	\
	$(foreach i,$(3),$($(i)_LINK:%=$(LINKXX_FLAG_LINK_LIBRARY)%$(LINKXX_FLAG_LIBRARY_EXTENSION))) \
	$(subst $(SPACE_FAKE),$(SPACE), \
		$(foreach i,$(3),$($(i)_LINK_DIR:%=$(LINKXX_FLAG_LINK_DIRECTORY)"%")) \
		$(if $(findstring $(COMPILATOR),gcc), \
			$(if $(foreach i,$(3),$($(i)_LINK_STATIC)), $(LINKXX_FLAG_LINK_LIBRARY_STATIC) $(foreach i,$(3),$($(i)_LINK_STATIC:%=$(LINKXX_FLAG_LINK_LIBRARY)%)),) \
			$(if $(foreach i,$(3),$($(i)_LINK_DYNAMIC)), $(LINKXX_FLAG_LINK_LIBRARY_DYNAMIC) $(foreach i,$(3),$($(i)_LINK_DYNAMIC:%=$(LINKXX_FLAG_LINK_LIBRARY)%)),) \
			\
			$(if $(foreach i,$(3),$($(i)_LINK_DIR)), $(LINKXX_FLAG_SAVE_LINK_PATH)$(subst $(SPACE),:,$(foreach i,$(3),$($(i)_LINK_DIR:%="%"))),)\
		) \
		$(if $(findstring $(COMPILATOR),visualc), \
			$(foreach i,$(3),$($(i)_LINK_STATIC:%=$(LINKXX_FLAG_LINK_LIBRARY_STATIC)%$(LINKXX_FLAG_LIBRARY_EXTENSION))) \
			$(foreach i,$(3),$($(i)_LINK_DYNAMIC:%=$(LINKXX_FLAG_LINK_LIBRARY_DYNAMIC)%$(LINKXX_FLAG_SHARED_LIBRARY_EXTENSION))) \
		) \
	)
	@$(ECHOLINEXX)

endef

