$(call onDebugMake,fichier 'libraries' inclu)

# ------------------------------------------------------------------------------
#        fichier contenant les repertoires standards pour les librairies
# ------------------------------------------------------------------------------
# ce fichier contient des fonctions pour permettre aux fichiers
# de configuration des libraries (fichiers contenus dans le
# repertoire "$(MAKE_INTERN_DIR)lib/") d'aller chercher dans les
# repertoires standards
# ------------------------------------------------------------------------------


LOCAL_LIB_DIRS ?=$(call exist,lib libs libraries tools)




# private_with_include : to test all possibilities with "basedir/include/libdir", "basedir/libdir/include", etc.
# 
# $(1) : dossier base (ex: /usr)
# $(2) : lib dir name (ex: gl)
# 
private_with_include = \
	$(foreach basedir,$(1),                                                    \
		$(basedir)                                                             \
		$(basedir)/include                                                     \
		$(foreach lib,$(2),                                                    \
			$(basedir)/$(lib)                                                  \
			$(basedir)/include/$(lib)                                          \
			$(basedir)/$(lib)/include                                          \
		)                                                                      \
	)



# ------------------------------------------------------------------------------
# standard_lib_incl : to search in standard directories (for include dir)
#
# $(1) : list of environment variables
# $(2) : list of name (for possible sub-directories)
#
standard_lib_incl=\
	$(2) $(LOCAL_LIB_DIRS) .                                                   \
	$(foreach d,$(2),                                                          \
		$(d)                                                                   \
		$(d)/include                                                           \
	)                                                                          \
	$(call private_with_include,                                               \
		. $(LOCAL_LIB_DIRS) $(PERSONNAL_LIBS_DIRS)                             \
		,                                                                      \
		$(2)                                                                   \
	)                                                                          \
	$(PERSONNAL_LIBS_DIRS)                                                     \
	$(foreach i,$(PERSONNAL_LIBS_DIRS),                                        \
		$(i)/include                                                           \
	)                                                                          \
	$(if $(WINDOWS),                                                           \
		$(call private_with_include,                                           \
			$(call convert_from_environementvariable,$(1))                     \
			,                                                                  \
			$(2)                                                               \
		)                                                                      \
		C:\WINDOWS\system32                                                    \
	)                                                                          \
	$(if $(LINUX)$(MACOSX),                                                    \
		$(call convert_from_environementvariable,PATH LD_LIBRARY_PATH)         \
		$(call private_with_include,                                           \
			$(call convert_from_environementvariable,$(1))                     \
			/usr                                                               \
			/usr/local                                                         \
			/opt                                                               \
			/opt/csw                                                           \
			/opt/local                                                         \
			/sw                                                                \
			,                                                                  \
			$(2)                                                               \
		)                                                                      \
	)                                                                          \
	$(if $(MACOSX),                                                            \
		$(foreach d,$(2),                                                      \
			~/Library/Frameworks/$(d).framework/Headers                        \
			/Library/Frameworks/$(d).framework/Headers                         \
			/System/Library/Frameworks/$(d).framework/Headers                  \
		)                                                                      \
	)
	


# private_with_lib : to test all possibilities with "basedir/lib/libdir", "basedir/libdir/lib", etc.
# 
# $(1) : dossier base (ex: /usr)
# $(2) : lib dir name (ex: gl)
# 
private_with_lib = \
	$(foreach basedir,$(1),                                                    \
		$(basedir)                                                             \
		$(basedir)/lib                                                         \
		$(basedir)/bin                                                         \
		$(foreach lib,$(2),                                                    \
			$(basedir)/$(lib)                                                  \
			$(basedir)/lib/$(lib)                                              \
			$(basedir)/$(lib)/lib                                              \
			$(basedir)/libs/$(lib)                                             \
			$(basedir)/$(lib)/libs                                             \
			$(basedir)/bin/$(lib)                                              \
			$(basedir)/$(lib)/bin                                              \
		)                                                                      \
	)


# ------------------------------------------------------------------------------
# standard_lib_link : to search in standard directories (for library link dir)
#
# $(1) : list of environment variables
# $(2) : list of name (for possible sub-directories)
#
standard_lib_link=\
	$(2) $(LOCAL_LIB_DIRS) .                                                   \
	$(PERSONNAL_LIBS_DIRS)                                                     \
	$(call private_with_lib,                                                      \
		.                                                                      \
		$(2)                                                                   \
		$(LOCAL_LIB_DIRS)                                                      \
		$(PERSONNAL_LIBS_DIRS)                                                 \
		$(call convert_from_environementvariable,$(1))                         \
		,                                                                      \
		$(2)                                                                   \
	)                                                                          \
	$(if $(WINDOWS),                                                           \
		$(call convert_from_environementvariable,PATH)                         \
		C:\WINDOWS\system32                                                    \
	)                                                                          \
	$(if $(LINUX)$(MACOSX),                                                    \
		$(call convert_from_environementvariable,PATH LD_LIBRARY_PATH)         \
		/usr/lib/w32api                                                        \
		/usr/shlib                                                             \
		$(call private_with_lib,                                                  \
			/usr                                                               \
			/usr/local                                                         \
			/usr/X11R6                                                         \
			/usr/openwin                                                       \
			/sw                                                                \
			/opt/local                                                         \
			/opt/csw                                                           \
			/opt                                                               \
			,                                                                  \
			$(2)                                                               \
		)                                                                      \
	)                                                                          \
	$(if $(MACOSX),                                                            \
		$(foreach d,$(2),                                                      \
			~/Library/Frameworks/$(d).framework/Libraries                      \
			/Library/Frameworks/$(d).framework/Libraries                       \
			/System/Library/Frameworks/$(d).framework/Libraries                \
		)                                                                      \
	)





