$(call onDebugMake,fichier 'configureConfig' inclu)



#-------------------------------------------------------------------------------
#                     Variables de configuration
#
# depend de la machine, des libraries utilisees, du systeme d'exploitation, etc.
#-------------------------------------------------------------------------------

# si le fichier de configuration n'a pas encore ete cree ou n'est pas a jour...
ifneq ($(REBUILD_CONFIG),)

$(call onDebugMake,REBUILD_CONFIG=true)

# le fichier contenant les variables pour rechercher dans les repertoires standards
include $(MAKE_INTERN_DIR)libraries.mk

VARCONFIG_TO_SAVE += LIBS_ALL LIBS_TOCHECK

#-------------------------------------------------------------------------------
# find_libs : declarer la liste des librairies utilisees
#
# $(1) : variable name
# $(2) : list of libraries
# example : $(call find_libs,MY_LIBS,SDL OPENGL)
#
find_libs = $(2ArgFunction)                                                    \
    $(info searching libraries files)                                          \
    $(foreach lib,$(2),                                                        \
        $(info $(SPACE) * searching "$(lib)" library files)                    \
		$(if $(call exist,$(BUILD_INTERN_DIR)libraries/$(lib).mk),             \
			$(eval include $(BUILD_INTERN_DIR)libraries/$(lib).mk)             \
		,                                                                      \
			$(eval $(lib)_LINK:=$(lib))                                        \
		)                                                                      \
	)                                                                          \
	$(eval $(1)_INCL     +=$(2:%=$$(%_INCL)))                                  \
	$(eval $(1)_LINK     +=$(2:%=$$(%_LINK)))                                  \
	$(eval $(1)_LINK_DIR +=$(2:%=$$(%_LINK_DIR)))                              \
	$(eval $(1)_FLAG     +=$(2:%=$$(%_FLAG)))                                  \
	$(eval VARCONFIG_TO_SAVE+=$(1)_INCL $(1)_LINK $(1)_LINK_DIR $(1)_FLAG)     \
	$(eval LIBS_ALL+=$(1) $(2))                                                \
	$(eval LIBS_TOCHECK+=$(2))                                                 \
	$(eval $(1)_INCL     :=$(sort $($(1)_INCL)))                               \
	$(eval $(1)_LINK     :=$(sort $($(1)_LINK)))                               \
	$(eval $(1)_LINK_DIR :=$(sort $($(1)_LINK_DIR)))


# ------------------------------------------------------------------------------
# find_libs_static  : declarer la liste des librairies statiques
# -Wl,-Bstatic -lsomestaticlib
#
# $(1) : variable name
# $(2) : list of libraries
# example : $(call find_libs_static,MY_LIBS,SDL OPENGL )
#
find_libs_static = $(2ArgFunction)                                             \
    $(info searching libraries files)                                          \
    $(foreach lib,$(2),                                                        \
        $(info $(SPACE) * searching "$(lib)" library files)                    \
		$(if $(call exist,$(BUILD_INTERN_DIR)libraries/$(lib).mk),             \
			$(eval include $(BUILD_INTERN_DIR)libraries/$(lib).mk)             \
		,                                                                      \
			$(eval $(lib)_LINK:=$(lib))                                        \
		)                                                                      \
	)                                                                          \
	$(eval $(1)_INCL_STATIC  += $(2:%=$$(%_INCL_STATIC)))                      \
	$(eval $(1)_LINK_STATIC  += $(2:%=$$(%_LINK_STATIC)))                      \
	$(eval $(1)_LINK_DIR     += $(2:%=$$(%_LINK_DIR)))                         \
	$(eval $(1)_FLAG         += $(2:%=$$(%_FLAG)))                             \
	$(eval VARCONFIG_TO_SAVE +=$(1)_INCL_STATIC $(1)_LINK_STATIC $(1)_LINK_DIR $(1)_FLAG) \
	$(eval LIBS_ALL+=$(1) $(2))                                                \
	$(eval LIBS_TOCHECK+=$(2))                                                 \
	$(eval $(1)_INCL_STATIC  :=$(sort $($(1)_INCL_STATIC)))                    \
	$(eval $(1)_LINK_STATIC  :=$(sort $($(1)_LINK_STATIC)))                    \
	$(eval $(1)_LINK_DIR     :=$(sort $($(1)_LINK_DIR)))


# ------------------------------------------------------------------------------
# find_libs_dynamic : declarer la liste des librairies dynamiques
# -Wl,-Bdynamic -lsomedynamiclib
#
# $(1) : variable name
# $(2) : list of libraries
# example : $(call find_libs_dynamic,MY_LIBS,SDL OPENGL )
#
find_libs_dynamic = $(2ArgFunction)                                            \
    $(info searching libraries files)                                          \
    $(foreach lib,$(2),                                                        \
        $(info $(SPACE) * searching "$(lib)" library files)                    \
		$(if $(call exist,$(BUILD_INTERN_DIR)libraries/$(lib).mk),             \
			$(eval include $(BUILD_INTERN_DIR)libraries/$(lib).mk)             \
		,                                                                      \
			$(eval $(lib)_LINK:=$(lib))                                        \
		)                                                                      \
	)                                                                          \
	$(eval $(1)_INCL_DYNAMIC +=$(2:%=$$(%_INCL_DYNAMIC)))                      \
	$(eval $(1)_LINK_DYNAMIC +=$(2:%=$$(%_LINK_DYNAMIC)))                      \
	$(eval $(1)_LINK_DIR     +=$(2:%=$$(%_LINK_DIR)))                          \
	$(eval $(1)_FLAG         +=$(2:%=$$(%_FLAG)))                              \
	$(eval VARCONFIG_TO_SAVE +=$(1)_INCL_DYNAMIC $(1)_LINK_DYNAMIC $(1)_LINK_DIR $(1)_FLAG) \
	$(eval LIBS_ALL+=$(1) $(2))                                                \
	$(eval LIBS_TOCHECK+=$(2))                                                 \
	$(eval $(1)_INCL_DYNAMIC :=$(sort $($(1)_INCL_DYNAMIC)))                   \
	$(eval $(1)_LINK_DYNAMIC :=$(sort $($(1)_LINK_DYNAMIC)))                   \
	$(eval $(1)_LINK_DIR     :=$(sort $($(1)_LINK_DIR)))


# ------------------------------------------------------------------------------
# private_declare_lib_incl : fonction interne de la fonction declare_lib_incl
# $(1) : name of the library
# $(2) : name of the main file of the library (which is in his root directory)
# $(3) : one possible directorie
#
# si il n'a pas encore ete rempli
private_declare_lib_incl = $(3ArgFunction)                                     \
    $(if $(call not,$($(1)_INCL)),                                             \
      $(if $(call exist,$(3)/$(2)),                                            \
        $(eval $(1)_INCL:=$(3))                                                \
        $(call onDebugMake, trouve: $(1)_INCL:=$(3))                           \
      ,)                                                                       \
    ,)

#      $(call onDebugMake, test if '$(3)/$(2)' file exist)                      \


# ------------------------------------------------------------------------------
# declare_lib_incl : declarer/trouver le chemin pour les fichiers d'include
#                    d'une librairie
#
# $(1) : name of the library
# $(2) : name of the main file of the library (which is in his root directory)
# $(3) : list of possible directories
# example : $(call declare_lib_incl, SDL, SDL.h, /usr/include/SDL /usr/include )
#
declare_lib_incl = $(3ArgFunction)                                             \
    $(info $(SPACE) ** include files)                                          \
	$(foreach d,$(3),$(call private_declare_lib_incl,$(1),$(strip $(2)),$(d)))          \
	$(eval VARCONFIG_TO_SAVE+=$(1)_INCL)                                       \
	$(foreach i,$(1),                                                          \
		$(eval $(i)_INCL :=$(sort $($(i)_INCL)))                               \
	)

#$(subst $(SPACE),,$(2))

# ------------------------------------------------------------------------------
# private_declare_lib_link : fonction interne de la fonction declare_lib_link
#
# $(1) : name of the library
# $(2) : name of the main file of the library (which is in his root directory)
# $(3) : one possible directory
#
#  lib*.a  ou lib*.so  --  sous linux
#  *.lib   ou *.dll    --  sous windows
#
private_declare_lib_link = $(3ArgFunction)                                     \
	$(if $(call exist,$(3)/$(LIBRARY_PREFIX)$(2)$(LIBRARY_EXTENSION)* $(3)/$(LIBRARY_PREFIX)$(2)$(SHARED_LIBRARY_EXTENSION)* $(3)/$(2)$(LIBRARY_EXTENSION)* $(3)/$(2)$(SHARED_LIBRARY_EXTENSION)*), \
		$(eval $(1)_LINK        += $(2))                                       \
		$(eval $(1)_LINK_DIR    += $(3))                                       \
		$(call onDebugMake, trouve: $(1)_LINK:=$(2) => $(1)_LINK_DIR += $(3))  \
	,)                                                                         \
	$(if $(call exist,$(3)/$(LIBRARY_PREFIX)$(2)$(LIBRARY_EXTENSION)* $(3)/$(2)$(LIBRARY_EXTENSION)*),        \
		$(eval $(1)_LINK_STATIC += $(2))                                       \
		$(eval $(1)_LINK_DIR    += $(3))                                       \
		$(call onDebugMake, trouve: $(1)_LINK_STATIC:=$(2) => $(1)_LINK_DIR += $(3))  \
	,)                                                                         \
	$(if $(call exist,$(3)/$(LIBRARY_PREFIX)$(2)$(SHARED_LIBRARY_EXTENSION)* $(3)/$(2)$(SHARED_LIBRARY_EXTENSION)*), \
		$(eval $(1)_LINK_DYNAMIC += $(2))                                      \
		$(eval $(1)_LINK_DIR     += $(3))                                      \
		$(call onDebugMake, trouve: $(1)_LINK_DYNAMIC:=$(2) => $(1)_LINK_DIR += $(3))  \
	,)

# $(call onDebugMake, test if '$(3)/$(LIBRARY_PREFIX)$(2)$(LIBRARY_EXTENSION)* $(3)/$(LIBRARY_PREFIX)$(2)$(SHARED_LIBRARY_EXTENSION)* $(3)/$(2)$(LIBRARY_EXTENSION)* $(3)/$(2)$(SHARED_LIBRARY_EXTENSION)*' files exists)                    \


# ------------------------------------------------------------------------------
# declare_lib_link : declarer/trouver les options de linkage d'une librairie
#
# $(1) : name of the library
# $(2) : name of the library file (which is in his root directory, without the prefix lib and the extension)
# $(3) : list of possible directories
# example : $(call declare_lib_link, SDL, SDL SDL-1.1, /usr/local/lib /usr/lib /sw/lib /opt/lib )
#
declare_lib_link = $(3ArgFunction)                                             \
    $(info $(SPACE) ** link files)                                             \
	$(foreach lib,$(2),                                                        \
		$(foreach dir,$(3),                                                    \
			$(call private_declare_lib_link,$(1),$(lib),$(dir))                \
		)                                                                      \
	)                                                                          \
	$(foreach i,$(1),                                                          \
		$(eval $(i)_LINK     :=$(sort $($(i)_LINK)))                           \
		$(eval $(i)_LINK_DIR :=$(sort $($(i)_LINK_DIR)))                       \
		$(eval $(i)_LINK_DYNAMIC :=$(sort $($(i)_LINK_DYNAMIC)))               \
		$(eval $(i)_LINK_STATIC :=$(sort $($(i)_LINK_STATIC)))                 \
	)                                                                          \
	$(eval VARCONFIG_TO_SAVE +=$(1)_LINK $(1)_LINK_STATIC $(1)_LINK_DYNAMIC $(1)_LINK_DIR )

# on ne peut pas l'afficher tout de suite...
#	$(call onDebugMake, resultat pour $(1) : $(1)_LINK:=$($(1)_LINK) -- $(1)_LINK_STATIC:=$($(1)_LINK_STATIC) -- $(1)_LINK_DYNAMIC:=$($(1)_LINK_DYNAMIC) -- $(1)_LINK_DIR:=$($(1)_LINK_DIR))


# ------------------------------------------------------------------------------
# declare_command : declarer/trouver une commande systeme specifique d'une
#                   librairie, par exemple pour qt
#
# $(1) : name of the command in the makefile
# $(2) : list of possible real command name
# $(3) : list of possible directories
#
declare_command = $(3ArgFunction)                                              \
    $(foreach command,$(2),                                                    \
      $(foreach dir,$(3) $(if $(LINUX)$(MACOS),/usr/bin) $(call convert_from_environementvariable,PATH), \
        $(if $(call exist,$(dir)/$(command) $(if $(WINDOWS),$(dir)/$(command).exe $(dir)/$(command).bat $(dir)/$(command).cmd) $(if $(LINUX),$(dir)/$(command).sh)), \
          $(eval $(1):=$(dir)/$(command))                                      \
        ,)                                                                     \
      )                                                                        \
    )                                                                          \
    $(eval VARCONFIG_TO_SAVE+=$(1))




else

$(call onDebugMake,REBUILD_CONFIG=false)

find_libs        = $(2ArgFunction)
declare_lib_incl = $(3ArgFunction)
declare_lib_link = $(3ArgFunction)
declare_command  = $(3ArgFunction)

include $(SAVE_CONFIG)

endif



