$(call onDebugMake,fichier 'unix/config' inclu)


#-------------------------------------------------------------------------------
#             fichier de config des Makefiles
# contient les elements dependants du syteme d'exploitation
#-------------------------------------------------------------------------------
# ce fichier contient :
# 1. les noms des commandes
# 2. fonctions
# 3. variables
# 4. parametres du compilateur
# 5. autres (date, couleur, ...)
#-------------------------------------------------------------------------------

# ne pas preciser...
#SHELL=/bin/sh

COMPILATOR ?= gcc

#_______________________________________________________________________________
# 1. le nom des commandes
#
# ccache g++ : compilation rapide avec ccache (utile lorsqu'on recompile souvent)
CPXX            := cp
CDXX            := cd
MKDIRXX         := mkdir -p
LSXX            := ls
SLASH           :=/
NO_END_LINE     :=\\
NO_ERROR_OUTPUT := 2>/dev/null
NO_OUTPUT       := >/dev/null 2>&1
NEXT_COMMAND    := ; 
AFF_NEXT_COMMAND:=$(NEXT_COMMAND)
CLEARXX         := clear
ECHOEMPTYXX     := echo
#$(subst (,\($(subst ),\),$(subst ',\',
#$(subst $$,\$$,
echo_convert    ="$(if $(1),$(subst ",\",$(1)),)"
echoxx          =echo $(call echo_convert,$(1))
echoexx         =echo -e $(call echo_convert,$(1))
PRINTFXX        := printf
ECHONEWLINEXX   := printf "\n"
ECHO2NEWLINESXX := printf "\n\n"
#-e #active l'interpretation des "\",
# notamment \033 pour faire de la couleur ou \n pour les retours charriots
FINDXX          := find
LATEXXX         := latex
LINKXX          := $(CCXX)
RMXX            := rm -f
RMDIRXX         := rm -r -f
TOUCHXX         := touch
GENDOC          := doxygen

# REDiriger la sortie standard et la sortie d'erreur vers un meme endroit :
# 2>&1

#_______________________________________________________________________________
# 2. functions
#

# convertir les slashs en fonction du systeme d'exploitation (on utilise ceux unix)
convert_slash_to_system   = $(1)
convert_slash_from_system = $(1)

convert_from_environementvariable = \
	$(subst :,$(SPACE),                                                    \
		$(foreach i,$(1),$(subst $(SPACE),$(SPACE_FAKE),$($(i))))      \
	)


# effectuer une operation mathematique
math = $(shell $(call echoxx,scale=0; $(1)) | bc )

# convertir les slashs en fonction du systeme d'exploitation (on utilise ceux unix)
CONVERT_SLASH =$(1)


# macros

# create a directory (without error if the directory already exist)
make_directoryxx = @$(MKDIRXX) $(1)

# $1 : file
# $2 : searched string
# return empty if $2 isn't found
find_in_file =$(shell grep "$(2)" $(1))


# $1 : files
# $2 : searched string
# return list of files which contain $2
find_in_files =$(foreach f,$(call exist,$(1)),$(if $(call find_in_file,$(f),$(2)),$(f)))



# $1 : string to repeat
# $2 : number of repeat
repeat_string =$(subst $(SPACE),,$(shell COUNTER=0;\
         while (( $$COUNTER < $2 )); do\
             $(call echoxx,$1);\
             (( COUNTER++ ));\
         done;\
  ))

# $3 : output variable
#repeat_string =$(eval $(3):=)$(foreach i,$(shell seq 1 $(2)),$(eval $(3):=$($3)$1))

#$(eval PROGRESSBAR_LEFT :=$(subst $(SPACE),,$(shell COUNTER=0;\
         while [ $$COUNTER -lt $(PROGRESSBAR_SIZELEFT) ]; do\
             $(call echoxx,=);\
             let COUNTER=COUNTER+1;\
         done;\
  )))
# OR
#  $(foreach i,$(shell seq 1 $(PROGRESSBAR_SIZELEFT)),$(eval PROGRESSBAR_LEFT:=$(PROGRESSBAR_LEFT)=))


#_______________________________________________________________________________
# 3. variables
#

EXECUTABLE_EXTENSION      :=
OBJECT_EXTENSION          :=.o
LIBRARY_PREFIX            :=lib
LIBRARY_EXTENSION         :=.a
SHARED_LIBRARY_EXTENSION  :=.so



#_______________________________________________________________________________
# 4. Parametres du compilateur
#
FLAG_SYSTEM         += $(CXX_FLAG_DEFINE)__SYSTEM_UNIX__



#_______________________________________________________________________________
# 5. autres
#

DATE = `date '+%Y-%m-%d--%Hh%M'`

ID_SESSION:=$(subst $(SPACE),_,$(strip $(USERNAME)_$(USER)_$(shell uname -mns)))

# le nom du dossier courant (pour avoir le chemin absolu utilise $(CURDIR))
CURRENTDIR = `basename $(CURDIR)`
# $(CURDIR) dans make ou $(PWD) au niveau systeme

ifndef NO_COLOR

#mise en page
BLACK  :=0
RED    :=1
GREEN  :=2
YELLOW :=3
BLUE   :=4
VIOLET :=5
CYAN   :=6
WHITE  :=7

TEXT       :=3
BACKGROUND :=4
BOLD       :=;1m
NORMAL     :=m
COLOR      :=\033[
# or \x1B[ or \E[
# code ASCII 27 => ESC


#-------------------------------------------------------------------------------
# COULEURS
#-------------------------------------------------------------------------------
TXTGREEN      :=$(COLOR)$(TEXT)$(GREEN)$(NORMAL)
TXTBROWN      :=$(COLOR)$(TEXT)$(YELLOW)$(NORMAL)
TXTCYAN       :=$(COLOR)$(TEXT)$(CYAN)$(NORMAL)
TXTDBLUE      :=$(COLOR)$(TEXT)$(BLUE)$(NORMAL)
TXTBLUE	      :=$(COLOR)$(TEXT)$(BLUE)$(BOLD)
TXTYELLOW     :=$(COLOR)$(TEXT)$(YELLOW)$(BOLD)
TXTRED        :=$(COLOR)$(TEXT)$(RED)$(NORMAL)
TXTREDBRIGHT  :=$(COLOR)$(TEXT)$(RED)$(BOLD)

#-------------------------------------------------------------------------------
# COULEURS des elements de l'affichage
#-------------------------------------------------------------------------------
COLORINIT     := $(COLOR)0m

# un makefile peut surcharger ces variables pour choisir ses couleurs
COLORBEGIN    ?=$(COLOR)$(BACKGROUND)$(BLACK);$(TEXT)$(WHITE)$(BOLD)
COLORTITLE    ?=$(TXTREDBRIGHT)
COLORTITLE3   ?=$(TXTBLUE)
COLORTITLE2   ?=$(TXTDBLUE)
COLORNOTES    ?=$(TXTRED)
COLORWARNING  ?=$(COLOR)$(BACKGROUND)$(RED);$(TEXT)$(WHITE)$(NORMAL)
COLORPROGRESS ?=$(TXTBLUE)




#-------------------------------------------------------------------------------
# POSITION de l'affichage dans la console
#-------------------------------------------------------------------------------
# $1 : y
# $2 : x
terminal_position =\033[$1;$2H 
TERMINAL_WIDTH  :=$(shell stty -a | grep columns | cut -d ' ' -f7 | tr -d ';')
TERMINAL_HEIGHT :=$(shell stty -a | grep rows | cut -d ' ' -f5 | tr -d ';')

ifeq ($(strip $(TERMINAL_WIDTH)),)
  TERMINAL_WIDTH:=80
endif

ifeq ($(strip $(TERMINAL_HEIGHT)),)
  TERMINAL_HEIGHT:=60
endif

endif

