$(call onDebugMake,fichier 'windows/config' inclu)


#-------------------------------------------------------------------------------
#             fichier de config
#-------------------------------------------------------------------------------
# ce fichier contient les configurations :
# 1. les noms des commandes
# 2. fonctions
# 3. variables
# 4. parametres du compilateur
# 5. autres (date, couleur, ...)
#-------------------------------------------------------------------------------

SHELL = cmd.exe

COMPILATOR ?=visualc


#_______________________________________________________________________________
# 1. le nom des commandes
#
CPXX            := copy /y
CDXX            := cd
MKDIRXX         := mkdir
LSXX            := dir /b/a-d/-W
SLASH           :=\\
NO_END_LINE     :=$(strip \ )
NO_ERROR_OUTPUT := 2>nul
NO_OUTPUT       := >nul 2>&1
NEXT_COMMAND    := && 
AFF_NEXT_COMMAND:= ^&^& 
CLEARXX         := cls
# "echo." seul genere une erreur...
ECHOEMPTYXX     := echo. 2>&1
ECHONEWLINEXX   := echo.
echoxx          =$(if $(1),echo $(1),$(ECHOEMPTYXX))
echoexx         =$(if $(1),echo $(1),$(ECHOEMPTYXX))
PRINTFXX        := echo
FINDXX          := dir /b/s/a-d/-W
#/b : abrege, sans aucune info; /s recursif; /a-d : sans les dossiers; /-W ignorer les options par defaut definis dans la variable d'environnement DIRCMD
#attention si ne trouve rien affiche un message...
LATEXXX         := latex
RMXX            := del /F/Q
#/F : force meme si en lecture seule; /Q : pas de demande de confirmation
RMDIRXX         := rd /S/Q
#/S : supprime recursivement sous-dossiers et sous-fichiers; /Q : pas de demande de confirmation
TOUCHXX         := copy /y nul
#/y : sans confirmation
GENDOC          := doxygen

# rediriger la sortie standard et la sortie d'erreur vers un meme endroit :
# commande > output_and_error.txt 2>&1


#_______________________________________________________________________________
# 2. functions
#

# effectuer une operation mathematique
math =$(shell cmd.exe /c "set /A $(1)")

# convertir les slashs en fonction du systeme d'exploitation (on utilise ceux unix)
convert_slash_to_system   =$(subst /,\,$(1))
convert_slash_from_system =$(subst \,/,$(1))

find_sourcefilesxx =$(filter-out %~,$(call convert_slash_from_system,$(shell $(FINDXX) $(1) $(NO_ERROR_OUTPUT) ) ) )

# on utilise "echo %var%" car $(var) fonctionne uniquement pour les variables
# d'environnement systeme et pas les variables d'environnement utilisateur
convert_from_environementvariable = \
  $(subst ;,$(SPACE),$(call convert_slash_from_system,                         \
    $(foreach i,$(1),                                                          \
	  $(subst $(SPACE),$(SPACE_FAKE),$(filter-out %$(i)%,$(shell $(call echoxx,%$(i)%)))) \
	)                                                                          \
  ))


# macros

# create a directory (without error if the directory already exist)
make_directoryxx =$(if $(call exist,$(1)), ,-@if not exist "$(1)/" $(MKDIRXX) "$(call convert_slash_to_system,$(1))" $(NO_ERROR_OUTPUT) )

# $1 : file
# $2 : searched string
# return empty if $2 isn't found
find_in_file =$(filter $(2),$(shell find /N "$(2)" $(1)))


# $1 : files
# $2 : searched string
# return list of files which contain $2
find_in_files =$(foreach f,$(call exist,$(1)),$(if $(call find_in_file,$(f),$(2)),$(f)))


# $1 : string to repeat
# $2 : number of repeat
repeat_string =$(subst $(SPACE),,$(strip $(shell cmd.exe /c "FOR /L %%i IN (1,1,$2) DO @echo $1")))

# $3 : output variable
#repeat_string =$(eval $(3):=)$(foreach i,$(shell cmd.exe /c "FOR /L %%i IN (1,1,$(2)) DO @echo _"),$(eval $(3):=$($(3))$1))

# for /F "delims=^" " %%A  in (fichier.txt) do echo %%A


#_______________________________________________________________________________
# 3. variables
#

EXECUTABLE_EXTENSION     :=.exe
OBJECT_EXTENSION         :=.obj
LIBRARY_PREFIX           :=
LIBRARY_EXTENSION        :=.lib
SHARED_LIBRARY_EXTENSION :=.dll





#_______________________________________________________________________________
# 4. parametres du compilateur
#
# remarque :
#  -O3 l'option pour le vers est "tiret-grand eau-trois"
FLAG_SYSTEM    = $(CXX_FLAG_DEFINE)__SYSTEM_WINDOWS__
# $(CXX_FLAG_DEFINE)_REENTRANT



#_______________________________________________________________________________
# 5. autres
#

DATE = `date '+%Y-%m-%d--%Hh%M'`

ID_SESSION:=$(subst $(SPACE),_,$(strip $(USERDOMAIN)_$(USERNAME)_$(COMPUTERNAME)_$(OS)_$(PROCESSOR_ARCHITECTURE)))


# le nom du dossier courant (pour avoir le chemin absolu utilise $(CURDIR))
CURRENTDIR = `basename $(CURDIR)`
#CURRENTDIR := $(subst \,/,$(shell cd ))
# $(CURDIR) dans make ou $(PWD) au niveau systeme


#-------------------------------------------------------------------------------
# COULEURS des elements de l'affichage
#-------------------------------------------------------------------------------
# pas de couleur sous windows


#-------------------------------------------------------------------------------
# POSITION de l'affichage dans la console
#-------------------------------------------------------------------------------
# pas de deplacement dans la console
terminal_position :=
# fixe
TERMINAL_WIDTH  :=80
TERMINAL_HEIGHT :=60

